//
//  Animation Example
//  guit GUI Toolkit
//  Copyright © 2019 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paristech.fr/~elc
//

#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;

class Animations : public GWindow {
public:
  Animations() : GWindow("#animations Aminations") {

    auto& anim = Anim(10'000)
    << "{pos: 10 10;   color: blue}   You like potato"
    << "{pos: 220 100; color: green}  And I like potahto"
    << "{pos: 50 150;  color: orange} You like tomato"
    << "{pos: 5 30;    color: pink}   And I like tomahto"
    << "{pos: 220 120; color: red}    Potato, potahto, tomato, tomahto!"
    << "{pos: 50 150;  color: purple} Let's call the whole thing off!"
    << "{pos: 10 10;   color: blue}   You like potato";

    auto& anim_area = VBox("{size:400 200; border:rounded; background:white}")
    << (Label("{font: bold 20 arial,sans-serif}") << anim);

    auto& anim_ctrl = HBox("{align: c c}")
    << (Button("{size:250 a; font: 16 arial,sans-serif} Move Text!")
        << on::checked   / [&]{anim.start();}
        << on::unchecked / [&]{anim.stop();}
        << on::mouseover / "Click Now!" / (Anim(700) << "color:pink" << "color:red")
        << on::checked   / "Stop Moving Text!" / (Anim(1500) << "align:start" << "align:end")
        );

    auto& left_part = VBox() << anim_area << anim_ctrl;


    GStrings steps{
      "background: red;    color: white; text: one",
      "background: yellow; color: blue;  text: two",
      "background: red;    color: white; text: three"
    };

    auto& postpone = Button("Postpone");
    postpone
    << on::action
    / [&]{ postpone = "<c:red>Starting in 2sec..."; }
    / CallWait(2000)
    / [&]{ postpone << "<c:blue> Action Now!"; }
    / CallWait(2000)
    / [&]{ postpone << "<c:red> Action Done!"; }
    / CallWait(1000)
    / [&]{ postpone << "Postpone"; };

    auto& right_part = VBox("{align:s x; gap:10; size:a a; flex:0 0}")
    << (Label("{border:rounded; align: c c; size: a 60; font:24 sans-serif}")
        << (Anim(5'000).start() << steps)
        )
    << (Button("  Anim on Enter  ") << on::mouseover / (Anim(1000) << steps))
    << (Button("  Anim on Press  ") << on::mousedown / (Anim(1000) << steps))
    << (Button("  Anim on Click  ") << on::action    / (Anim(1000,1) << steps))
    << postpone;


    *this
    << left_part
    << right_part;
  }
};


GWindow& Examples::makeAnimations() {
  return *new Animations;
}
