//
//  Guit GUI Toolkit Examples.
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include <gstatechart.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;


class Clock : public GWindow {
public:
  Clock() : GWindow("#clock Clock") {

    // Clock timer: fired every second indefinitely
    auto& timer = Timer(1000);

    // timer.addBeat(n, limit) creates a GInt that is increased by 1 when timer
    // has been fired 'n' times and that it is reset to 0 when it reaches 'limit'
    auto& seconds = timer.addBeat(1, 60);
    auto& minutes = timer.addBeat(60, 60);
    auto& hours   = timer.addBeat(60*60, 24);

    // state machine and states for setting the hours and minutes.
    auto& sc = StateChart();
    auto& running = sc.addState();
    auto& sethour = sc.addState();
    auto& setmin  = sc.addState();

    auto& digitbox = HBox
    << (Label(".digits")
        << hours
        << +sethour / GColor::orange  // orange color when sethours is entered
        )
    << (Label(".digits")
        << minutes
        << +setmin / GColor::orange
        )
    << (Label(".digits")
        << seconds
        );

    auto& buttonbox = VBox(".buttonbox")
    << -running / (Button(" + ")      // shown only if not in running state
                   << Guard(+sethour) / [&]{hours++;}   // increase hours when clicked if sethour entered
                   << Guard(+setmin)  / [&]{minutes++;}
                   )
    << (Button()
        << +running / "Set "   // displays "Set" when running state entered
        << +sethour / "Hrs."
        << +setmin  / "Min."
        // transition performed when button is clicked
        << (running >> sethour) / [&]{timer.stop();}
        << (sethour >> setmin)
        << (setmin  >> running) / [&]{timer.start();}
        )
    << -running / (Button(" - ")
                   << Guard(+sethour) / [&]{hours--;}
                   << Guard(+setmin)  / [&]{minutes--;}
                   );
    
    *this
    << on::show / [&]{timer.start();}
    << on::hide / [&]{timer.stop();}
    << digitbox
    << buttonbox;
    
    sc.start(running);    // starts state machine
  }
};


GWindow& Examples::makeClock() {
  return *new Clock();
}
