//
//  Guit GUI Toolkit Examples
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;

class PiesAndGrids : public GWindow {
  GChoice choice;
  GText  chosen_text;
  GColor chosen_color;

public:
  PiesAndGrids() : GWindow("#pies-grids Pies and Grids")
  {
    auto clicked = [&](GEvent& e) {
      if (auto* item = e.chosen()) {
        chosen_text = item->text();
        chosen_color = item->prop<GBackground>();
      }
    };

    // Items can be added directly by using the << operator. However, sometimes
    // it is more convenient to use a vector of strings (GStrings). This vector
    // can be added to as many boxes (or gadgets deriving from GBox) as needed.
    //
    // Item attributes (such as the color) can be specified inside {} in the same
    // way as in CSSs. The {} clause must be after CSS classes and id (if provided)
    // and before the item's title, eg: ".myclass #myid {....} mytitle)"
    //
    // The items of a box can also be specified in a CSS through the 'items' prop,
    // eg, if the box' id is #mybox, the CSS rule is:
    //     #mybox {items: one, two, three}
    // Attributes can be specified the same way:
    //     #mybox {items: {color:red} one, {color:blue} two, {color:green} three}

    GStrings items{
      "{background:red} one",
      "{background:yellow} two",
      "{background:green} three",
      "{background:blue} four",
      "{background:orange} five"
    };

    auto& piemenu = PieMenu(".items {size:120 120}")  // needs size!
    << items
    << clicked;

    auto& gridmenu = Menu(".items {flow:grid 2; background:none; border:none}")
    << items
    << clicked;

    // boxes that open pie or grid menus
    auto& menu_panel = HBox()
    << (HBox(".menupane")
        << (Button("Pie Menu") << piemenu)
        )
    << (HBox(".menupane")
        << (Button("Grid Menu") << gridmenu)
        )
    << (HBox(".menupane")
        << piemenu
        << "Pie Menu"
        )
    << (HBox(".menupane")
        << gridmenu
        << "Grid menu"
        );

    // Same as above except that the items have a numerical *value* because they
    // are added to a Chart (see below). The value and the text can be provided
    // in various ways as shown below

    GStrings valitems{
      "{background:red} 80 one",
      "{background:yellow} 30 two",
      "{background:green; value:70} three",
      "{background:blue; value:90} four",
      "{background:orange; value:50; text:five}"
    };

    // Charts that display items as a pie, a row, a column or a grid.
    // A major difference with other boxes is that the items must have a
    // numerical *value*. This value specifies the size of each item relatively
    // to its siblings.

    auto& chart_panel = HBox()
    << (ChartBox(".pane {flow:pie;}")
        << valitems
        << choice
        << clicked
        )
    << (ChartBox(".pane {flow:column}")
        << valitems
        << choice
        << clicked
        )
    << (ChartBox(".pane {flow:row}")
        << valitems
        << choice
        << clicked
        )
    << (ChartBox(".pane {flow:grid 2}")
        << valitems
        << choice
        << clicked
        );

    *this
    << menu_panel
    << chart_panel
    << (Label("#chosen") << chosen_color << ("Chosen: " + ~chosen_text));
  }
};

GWindow& Examples::makePiesAndGrids() {
  return *new PiesAndGrids;
}
