//
//  Gadgets that open a menu of options
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_ComboBox_hpp
#define Guit_ComboBox_hpp
#include <gchoicebox.hpp>
namespace guit {

/// Displays a selector (Button) that opens a menu of options.
/// The openend menu works as a GListBox. Adding a child to this gadget adds its to the menu.
/// Adding a string adds an item to the menu.
class GOptionBox : public GChoiceBox {
protected:
  GOptionBox(GadgetType&, GString const& css_and_title);

public:
  ~GOptionBox();
  GOptionBox(GString const& css_and_title = "");
  GOptionBox& clone(GClone const&) override;

  /// Returns the Button that opens the menu.
  GItem& selector();

  /// Returns the associated Menu.
  GMenu& menu();

  using GBox::add;
  void add(Gadget* child, int index = -1) override;
  void add(Gadget& child, int index = -1) override {GOptionBox::add(&child, index);}

  using GBox::remove;
  bool remove(Gadget* child) override;
  bool removeAt(int index) override;
  void removeChildren(Remove mode = Remove::InContent) override;

  //child(), childindex() etc. A REVOIR !!!!!

  using GType = GadgetType_<GOptionBox,GChoiceBox>;
  static GType& Type();
  GType& type() const override {return Type();}

  /// @internal.
  /// @{
  gptr<GItem> selector_;
  gptr<GMenu> menu_;
  void init(GRender* parrender) override;
  void initChildren(GRender* parrender) override;
  void initClone(Gadget& from, GClone const&) override;
  void setChoiceImpl(GChoice& c, bool add_to_list) override;
  GBox& itemBox() override;
  GBox const& itemBox() const override;
  virtual void changeCB(GEvent&);
  /// @}
};

GOptionBox& OptionBox(GString const& css_and_title = "");

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

/// Displays a text field that opens a menu of options.
/// Works as a GOptionBox.
class GComboBox : public GOptionBox {
protected:
  GComboBox(GadgetType&, GString const& css_and_title);

public:
  GComboBox(GString const& css_and_title = "");
  GComboBox& clone(GClone const&) override;

  /// Returns the TextField.
  GTextItem& selector();

  using GType = GadgetType_<GComboBox,GOptionBox>;
  static GType& Type();
  GType& type() const override {return Type();}
};

GComboBox& ComboBox(GString const& css_and_title = "");

}
#endif
