//
//  Spin Item
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gevent.hpp>
#include <gtrigger.hpp>
#include <gspinitem.hpp>
#include <gicon.hpp>
#include <gfloat.hpp>
#include <ggraphics.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_GADGET(SpinItem)

GSpinItem::~GSpinItem() {}

GSpinItem::GSpinItem(GadgetType& t, GString const& spec) : GTextItem(spec) {
  numvalue_ = new GFloat();
  //incr_ = new GFloat(1.f);
  icon_ = new GIcon(GIcon::UpDownSymbol, GColor::guitblue, GColor::orange);
  *this
  << numvalue_
  << icon_
  << on::mousedown / [this](GEvent& e){mouseCB(e.asMouseEvent());}
  << on::mouseup / [this]{icon_->value_.area = 0;};
}

GSpinItem::GSpinItem(GString const& spec)
: GSpinItem(Type(), spec) {}

GSpinItem& GSpinItem::clone(GClone const& c) {
  auto* g = new GSpinItem;
  g->initClone(*this, c);
  return *g;
}

void GSpinItem::mouseCB(GMouseEvent& e) {
  //if (e.x() > w() - GPrefs::iconWidth()) {
  if (e.x() > w() - h()) {
    if (e.y() < h() / 2) {
      icon_->value_.area = 1;
      numvalue_->increaseF(incr_);
    }
    else {
      icon_->value_.area = 2;
      numvalue_->increaseF(-incr_);
    }
  }
}

void GSpinItem::addProp(GProp& prop, bool add_to_list) {
  if ((prop.role() == GPropType::Number || prop.role() == GPropType::Choice)
      && prop.isMutable()) {
    numvalue_ = prop.toNumber();
  }
  GTextItem::addProp(prop, add_to_list);
}

GNumber& GSpinItem::numProp() {return *numvalue_;}

GTextItem& GSpinItem::operator=(float val) {
  numvalue_->setF(val);
  return *this;
}

int GSpinItem::intValue() const {return int(numvalue_->floatValue());}
float GSpinItem::floatValue() const {return numvalue_->floatValue();}

float GSpinItem::min() const {return numvalue_->minF();}
float GSpinItem::max() const {return numvalue_->maxF();}

GSpinItem& GSpinItem::setRange(float min, float max) {
  numvalue_->setRangeF(min, max);
  return *this;
}

}
