//
//  Text Gadgets
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gevent.hpp>
#include <gtrigger.hpp>
#include <gbind.hpp>
#include <gtextitem.hpp>
#include <gbox.hpp>
#include <gexpr.hpp>
#include <gbool.hpp>
#include <gint.hpp>
#include <gfloat.hpp>
#include <gborder.hpp>
#include <gcontrol.hpp>
#include <gstyle.hpp>
#include <gadgetstyle.hpp>
#include <grender.hpp>
#include <gtextrender.hpp>
#include <gnative.hpp>
#include <gstring.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_GADGET(TextItem)
GIMPLEMENT_GADGET_ALL(TextField, GTextItem)
GIMPLEMENT_GADGET_TYPE(TextArea)

// bug etrange: si la chaine est vide le textaerea disparait !!!!
GTextArea::GTextArea(GString const& spec)
: GTextItem( (spec.empty() || spec== " ") ? "  " : spec) {}

GTextArea& GTextArea::clone(GClone const& c) {
  auto* g = new GTextArea;
  g->initClone(*this, c);
  return *g;
}

GTextArea& TextArea(GString const& spec) {return *new GTextArea(spec);}

// - - - - -

GTextAttr::GTextAttr(size_t pos, size_t len)
: pos_(pos), len_(len) {}

void GTextAttributes::clear() {
  for (auto& it : list_) delete it;
  list_.clear();
}

void GTextAttributes::add(GTextAttr& attr) {
  list_.push_back(&attr);
}

GTextAttr* GTextAttributes::find(size_t pos) const {
  GTextAttr* last{};
  for (auto* it : list_) {
    if (it->pos_ <= pos) last = it;
    if (it->pos_ >= pos) break;
  }
  if (last && pos < last->pos_ + last->len_) {
    return last;
  }
  return nullptr;
}

GTextAttr* GTextAttributes::find(GObject* obj) const {
  for (auto* it : list_) {
    if (it->data_.get() == obj) return it;
  }
  return nullptr;
}

void GTextAttributes::adjust(size_t pos, ssize_t delta)  {
  //cerr << "adjust " << pos <<" "<< delta<<endl;
  for (size_t k = 0; k < list_.size(); ++k) {
    if (pos < list_[k]->pos_) {
      list_[k]->pos_ += delta;
    }
    else if (pos <= list_[k]->pos_ + list_[k]->len_) {
      list_[k]->len_ += delta;
    }
  }
}

void GTextItem::setAttributes(GTextAttributes* alist) {
  delete attrlist_;
  attrlist_ = alist;
}

// - - - - - - - - - - -

GTextItem::~GTextItem() {}

GTextItem::GTextItem(GadgetType& t, GString const& spec) : GItem(t) {
  eventmask_ = eventmask_ | GEventType::MouseButton | GEventType::Key;
  *this << on::dblclick / [this]{selectAllTextCB();};

  addProp(*new GText, true);   // inits value_, must always be created!

  if (!spec.empty()) {
    GString title;
    GadgetStyle::initStyle(*this, spec, title);
    if (!title.empty()) value_->set(title);
  }
}

GTextItem::GTextItem(GString const& spec)
: GTextItem(Type(), spec) {}


GTextItem& GTextItem::clone(GClone const& c) {
  auto* g = new GTextItem;
  g->initClone(*this, c);
  return *g;
}

GText& GTextItem::operator()() {return *value_;}

GString GTextItem::text() const {return value_->value();}
//GString const& GTextItem::value() const {return value_->value();}

GString GTextItem::stringValue() const {return value_->value();}
int GTextItem::intValue() const {return int(value_->floatValue());}
float GTextItem::floatValue() const {return value_->floatValue();}

GTextItem& GTextItem::operator=(GString const& text) {
  value_->set(text, false);
  return *this;
}

GTextItem& GTextItem::operator=(float val) {
  GString str;
  gconvert(val, str);
  return operator=(str);
}

void GTextItem::selectAllTextCB() {
  ctrl_.selbegin = 0;
  ctrl_.selend = ssize_t(value_->size());
}

void GTextItem::setValue(GString const& text) {
  value_->set(text, false);
}

void GTextItem::add(GString const& str) {
  value_->set(str, false);
}

void GTextItem::addProp(GProp& prop, bool add_to_list) {
  switch (prop.role()) {

    case GPropType::Text:
      if (!prop.isMutable()) value_->set(prop.stringValue(), false);  // just set
      else {  // replace prop
        addProp2(prop, add_to_list);
        value_ = prop.toText();
        prop.toMutableProp()->startNotify(*this, [](Gadget& g){g.updateLayout();});
      }
      updateLayout();
      return;
      
    case GPropType::Expr:
    case GPropType::ExprWrapper: {
      GExprWrapper* w{};
      if (auto* e = prop.toExpr()) w = new GExprWrapper(*e);
      else if (!(w = prop.toExprWrapper())) return;
      w->bindWith(*value_);
      addProp2(*w, add_to_list);
      updateLayout();
    } return;

    case GPropType::Number:
      if (!prop.isMutable()) {
        value_->set(prop.stringValue(), false);  // just set
      }
      else {  // bind prop
        addProp2(prop, add_to_list);
        if (auto* pi = prop.toInt()) gsync(*value_, *pi, -1);
        else if (auto* pf = prop.toFloat()) gsync(*value_, *pf, -1);
        prop.toMutableProp()->startNotify(*this, [](Gadget& g){g.updateLayout();});
      }
      updateLayout();
      return;
      
    default:
      GItem::addProp(prop, add_to_list);
      return;
  }
}

void GTextItem::pack(GRender* parr) {
  GRender r(*this, parr);
  font_ = r.font_;
  margin_ = r.margin_;

  if (!isMultiLine()) return;

  GBox* par = getParent();
  if (!par) return;
  
  if (!tcache_) tcache_ = new GTextCache;

  //if (rowFlexRef(this) == 0 || columnFlexRef(this) == 0) return;  // !!!!!!!!
  auto dim = GTextRender::layoutText(r.font_, modes_.richText, value_->value(),
                                     par->childrenArea(*parr).w,
                                     attrlist_, tcache_);
  if (r.margin_)
    frame_.h = dim.h + r.margin_->top + r.margin_->bottom; // + 20;
  else
    frame_.h = dim.h; // + 20;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

//ssize_t GTextItem::caret() const {return byteToChar(caret_);}

ssize_t GTextItem::posToIndex(ssize_t charpos) const {
  return strPosToIndex(value_->value(), charpos, modes_.richText);
}

ssize_t GTextItem::indexToPos(ssize_t bytepos) const {
  return strPosToIndex(value_->value(), bytepos, modes_.richText);
}

void GTextItem::findCaretRowColumn(ssize_t index, int& line, int& column) {
  if (!modes_.multiLine) {
    line = 0;
    column = int(indexToPos(index));
  }
  else if (!tcache_ || !tcache_->rows_) {
    line = column = 0;
  }
  else {
    int count = 0;
    line = -1;
    for (auto& it : *tcache_->rows_) {
      if (ssize_t(it.start_) <= index
          && ssize_t(it.start_ + it.length_) >= index) {
        line = count;
        break;
      }
      count++;
    }
    if (line < 0) {line = column = 0;}
    else column = int(indexToPos(index)
                      - indexToPos((*tcache_->rows_)[line].start_));
  }
}

void GTextItem::caretRowColumn(int& line, int& column) {
  findCaretRowColumn(ctrl_.caret, line, column);
}

int GTextItem::caretColumn() {
  int line{}, column{};
  findCaretRowColumn(ctrl_.caret, line, column);
  return column;
}

void GTextItem::moveCaret(GPoint const& wpos) {
  ssize_t prev_caret = ctrl_.caret;
  ctrl_.caret = findCaretIndex(wpos);
  if (ctrl_.caret != prev_caret) {
    ctrl_.selend = ctrl_.caret;
    //cerr << "SEL " <<ctrl_.selbegin  << " " <<ctrl_.selend << endl;
    GEvent e(GTrigger::caret, GCore::elapsedTime());
    doFire(e);
  }
}

void GTextItem::setCaretIndex(ssize_t ix) {       // should scroll if needed
  ssize_t prev_caret = ctrl_.caret;
  ctrl_.selbegin = ctrl_.selend = ctrl_.caret = (ix);

  if (ctrl_.caret != prev_caret) {
    GEvent e(GTrigger::caret, GCore::elapsedTime());
    doFire(e);
  }
  update();
}

void GTextItem::setCaretColumn(int column) {
  setCaretRowColumn(0, column);
}

void GTextItem::setCaretRowColumn(int line, int column) {
  if (!modes_.multiLine) {
    setCaretIndex(posToIndex(column));
  }
  else if (tcache_ && line >= 0 && size_t(line) < tcache_->rows_->size()) {
    auto& r = (*tcache_->rows_)[line];
    if (column >= ssize_t(r.length_)) column = int(r.length_ - 1);         // !!!! PAS BOn !!!
    if (column < 0) column = 0;
    setCaretIndex(posToIndex(column) + r.start_);
  }
}

void GTextItem::setCaret(GPoint const& wpos) {
  setCaretIndex(findCaretIndex(wpos));
}

ssize_t GTextItem::findCaretIndex(GPoint const& caretXY) {
  float x = frame_.x;
  float y = frame_.y;
  if (margin_) {
    x += margin_->left;
    y += margin_->top;
  }

  if (!isMultiLine()) {
    return findCaretIndexInRow(ctrl_.offset, value_->size(), caretXY.x - x);
  }
  else {
    if (!tcache_) return 0;

    auto caretY = caretXY.y - y;
    for (auto& it : *tcache_->rows_) {
      if (it.y_ <= caretY && it.y_ + it.h_ > caretY) {
        return findCaretIndexInRow(it.start_, it.start_ + it.length_,
                                   caretXY.x - x);
      }
    }
    return value_->empty() ? 0 : (value_->size() - 1); // not found
  }
}

ssize_t GTextItem::findCaretIndexInRow(ssize_t begin, ssize_t end, float caretX) {
  float prevw = 0.f;
  ssize_t ix = begin;

  while (ix < end) {
    ssize_t ix2 = value_->nextIndex(ix, modes_.richText);
    if (ix == ix2) return ix;
    auto w = font_.textSize(value_->substr(begin, ix2-begin),
                            modes_.richText, attrlist_).w;
    if (w > caretX) {
      return (w - caretX < caretX - prevw) ? ix2 : ix;
    }
    ix = ix2;
    prevw = w;
  }
  return end; // end car apres le dernier
}

void GTextItem::deleteChar(bool before) {
  if (ctrl_.caret < 0 || value_->empty()) {
    ctrl_.selbegin = ctrl_.selend = ctrl_.caret = 0;
    return;
  }
  if (deleteSelection()) return;
  if (ctrl_.caret <= 0) {
    ctrl_.selbegin = ctrl_.selend = ctrl_.caret = 0;
    return;
  }
  ssize_t remainpos = before ? ctrl_.caret : value_->nextIndex(ctrl_.caret, isRichText());
  ctrl_.selbegin = ctrl_.selend = ctrl_.caret;
  if (before) moveCaretLeft();

  // only if the char is actually removed
  if (value_->erase(ctrl_.caret, size_t(remainpos - ctrl_.caret))) {
    if (attrlist_) attrlist_->adjust(ctrl_.caret, -(remainpos - ctrl_.caret));
    GEvent e(GTrigger::change, GCore::elapsedTime());
    doFire(e);
  }
  else moveCaretRight(1, true);
}

void GTextItem::deleteEnd() {
  if (!modes_.multiLine) return;
  if (ctrl_.caret < 0 || value_->empty()) {
    ctrl_.selbegin = ctrl_.selend = ctrl_.caret = 0;
    return;
  }
  ctrl_.selbegin = ctrl_.selend = ctrl_.caret; // !!
  // only if the char is actually removed
  if (value_->erase(size_t(ctrl_.caret))) {
    if (attrlist_) attrlist_->adjust(ctrl_.caret, size_t(value_->size() - ctrl_.caret));
    GEvent e(GTrigger::change, GCore::elapsedTime());
    doFire(e);
  }
}

bool GTextItem::deleteSelection() {
  if (ctrl_.caret < 0 || value_->empty()|| ctrl_.selbegin == ctrl_.selend) {
    return false;
  }
  ssize_t from{}, to{};
  if (ctrl_.selbegin < ctrl_.selend) {
    from = ctrl_.selbegin;
    to = ctrl_.selend;
  }
  else {
    from = ctrl_.selend;
    to = ctrl_.selbegin;
  }
  if (value_->erase(from, size_t(to - from))) {
    if (attrlist_) attrlist_->adjust(to, -(to - from));
    GEvent e(GTrigger::change, GCore::elapsedTime());
    doFire(e);
  }
  ctrl_.caret = from;
  ctrl_.selbegin = ctrl_.selend = 0;
  return true;
}

void GTextItem::insertText(GString const& text) {
  if (text.empty() || (std::iscntrl(text[0]) && text[0]!='\n')) return;
  deleteSelection();

  auto count = strIndexToPos(text, -1, isRichText());
  if (ctrl_.caret > ssize_t(value_->size())) ctrl_.caret = value_->size();
  ctrl_.selbegin = ctrl_.selend = ctrl_.caret;

  // only if the char is actually inserted
  if (value_->insert(ctrl_.caret, text)) {
    if (attrlist_) attrlist_->adjust(ctrl_.caret, text.size());
    moveCaretRight(count, true);
    GEvent e(GTrigger::change, GCore::elapsedTime());
    doFire(e);
  }
}

void GTextItem::copyOrCut(bool cut) {
  if (ctrl_.caret < 0 || value_->empty() || ctrl_.selbegin == ctrl_.selend) {
    return;
  }
  size_t from{}, to{};
  if (ctrl_.selbegin < ctrl_.selend) {
    from = ctrl_.selbegin;
    to = ctrl_.selend;
  }
  else {
    from = ctrl_.selend;
    to = ctrl_.selbegin;
  }
  GString s = value_->substr(from, to-from);
  GCore::natApp()->setClipboard(s);
  if (cut) {  // cut
    deleteSelection();
    GEvent e(GTrigger::cut, GCore::elapsedTime());
    doFire(e);
  }
  else {   // copy
    GEvent e(GTrigger::copy, GCore::elapsedTime());
    doFire(e);
  }
}

void GTextItem::paste() {
  GString s;
  if (GCore::natApp()->getClipboard(s)) {
    insertText(s);
    GEvent e(GTrigger::paste, GCore::elapsedTime());
    doFire(e);
  }
}

void GTextItem::moveCaretRight(ssize_t count, bool scroll_one) {
  auto richtext = isRichText();

  if (ctrl_.caret >= ssize_t(value_->size())) {
    ctrl_.caret = value_->size();
    ctrl_.selbegin = ctrl_.selend = 0;
    return;
  }
  for (int k = 0; k < count; ++k) {
    ctrl_.caret = value_->nextIndex(ctrl_.caret, richtext);
  }
  ctrl_.selbegin = ctrl_.selend = 0;

  if (!modes_.multiLine) {  // scroll text (change offset) if needed
    GRender r(*this, nullptr);  // !! il manque parent_render !!!

    float xmax = frame_.w - (r.margin_ ? r.margin_->right : 0.f);
    float left_margin = r.margin_ ? r.margin_->left : 0.f;
    float xcaret = left_margin
    + r.font_.textSize(value_->value().substr(ctrl_.offset,
                                              ctrl_.caret - ctrl_.offset),
                       richtext, attrlist_).w;
    // caret out of box => change offset to scroll string
    if (xcaret > xmax - GPrefs::caretEndDist()) {
      if (scroll_one) {
        ctrl_.offset = value_->nextIndex(ctrl_.offset, richtext);
      }
      else for (int i = 0; i < GPrefs::caretScroll(); ++i) {
        ctrl_.offset = value_->nextIndex(ctrl_.offset, richtext);
        xcaret = left_margin
        + r.font_.textSize(value_->value().substr(ctrl_.offset,
                                                  ctrl_.caret - ctrl_.offset),
                           richtext, attrlist_).w;
        if (xcaret < xmax - GPrefs::caretEndDist()) break;
      }
    }
  }
  GEvent e(GTrigger::caret, GCore::elapsedTime());
  doFire(e);
}

void GTextItem::moveCaretLeft() {
  if (ctrl_.caret <= 0) {
    ctrl_.caret = 0;
    ctrl_.selbegin = ctrl_.selend = 0;
    return;
  }
  ctrl_.caret = value_->previousIndex(ctrl_.caret, isRichText());
  if (ctrl_.caret < 0) ctrl_.caret = 0;
  ctrl_.selbegin = ctrl_.selend = 0;
  if (!modes_.multiLine && ctrl_.caret < ctrl_.offset) {
    for (int i = 0; i < GPrefs::caretScroll(); ++i) {
      ctrl_.offset = value_->previousIndex(ctrl_.offset, isRichText());
    }
  }
  GEvent e(GTrigger::caret, GCore::elapsedTime());
  doFire(e);
}

void GTextItem::moveCaretUp() {
  if (!modes_.multiLine) return;
  ctrl_.selbegin = ctrl_.selend = 0;
  int line = 0, column = 0;
  caretRowColumn(line, column);
  setCaretRowColumn(line-1, column);
}

void GTextItem::moveCaretDown() {
  if (!modes_.multiLine) return;
  ctrl_.selbegin = ctrl_.selend = 0;
  int line = 0, column = 0;
  caretRowColumn(line, column);
  setCaretRowColumn(line+1, column);
}

bool GTextItem::keyCB(GKeyEvent& e) {
  auto kcode = e.key();

  if (kcode == GEvent::LeftKey) {
    moveCaretLeft();
    return true;
  }
  else if (kcode == GEvent::RightKey) {
    moveCaretRight(1, false);
    return true;
  }
  else if (kcode == GEvent::UpKey) {
    moveCaretUp();
    return true;
  }
  else if (kcode == GEvent::DownKey) {
    moveCaretDown();
    return true;
  }
  else if (kcode == GEvent::BackspaceKey) {
    deleteChar(true);
    return true;
  }
  else if (kcode == GEvent::DeleteKey) {
    deleteChar(false);
    return true;
  }
  else if (kcode == GEvent::ReturnKey || kcode == GEvent::EnterKey) {
    if (modes_.multiLine) insertText("\n");
    else {
      GEvent e2(on::action, GCore::elapsedTime());
      doFire(e2);
    }
    return true;
  }
  else if (e.ctrlMod()) {
    switch (kcode) {
      case 'd':
      case 'D':
        deleteChar(false);
        return true;
      case 'h':
      case 'H':
        deleteChar(true);
        return true;
      case 'k':
      case 'K':
        deleteEnd();
        return true;
      default:
        return false;
    }
  }
  else if (e.metaMod()) {
    switch (kcode) {
      case 'v':
      case 'V':
        paste();
        return true;
      case 'x':
      case 'X':
        copyOrCut(true);
        return true;
      case 'c':
      case 'C':
        copyOrCut(false);
        return true;
      default:
        return false;
    }
  }
  else {
    if (e.text().empty()) return false;
    else {
      insertText(e.text());
      return true;
    }
  }
}

}
