//
// CSS internals: Value and Item props
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef GuitItems_hpp
#define GuitItems_hpp
#include <gprop.hpp>
namespace guit {

/// CSS Value.
/// Stores the value of Gadgets having a value (see Gadget::hasValue()) in a CSS.
class GValue : public GProp {
public:
  GValue(GString const& value = "") : value_(value) {}

  using GType = GPropType_<GValue,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

  bool onAdd(Gadget*) override;

  GString value_;
};


/// CSS Items of a GBox.
/// Stores items of GBox Gadgets in a CSS.
class GItems : public GProp {
public:
  GItems(GString const& items = "");

  bool onAdd(Gadget*) override;
  void apply(Gadget*, GRender&, Specif) final;
  static GString parseClass(GString const& items, size_t & from);

  using GType = GPropType_<GItems,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

  GString items_;
  GString itemclass_;
};

}
#endif
